@@ -21,6 +21,7 @@
 #include "msgpack/object.hpp"
 #include "msgpack/zone.h"
 #include <cstdlib>
+#include <memory>
 #include <vector>
 
 
@@ -38,6 +39,9 @@ public:
 
 	void push_finalizer(void (*func)(void*), void* data);
 
+	template <typename T>
+	void push_finalizer(std::auto_ptr<T> obj);
+
 	void clear();
 
 	
@@ -139,6 +143,15 @@ inline void zone::push_finalizer(void (*func)(void*), void* data)
 	}
 }
 
+template <typename T>
+inline void zone::push_finalizer(std::auto_ptr<T> obj)
+{
+	if(!msgpack_zone_push_finalizer(this, &zone::object_destructor<T>, obj.get())) {
+		throw std::bad_alloc();
+	}
+	obj.release();
+}
+
 inline void zone::clear()
 {
 	msgpack_zone_clear(this);
